/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ProvidingPhenotype;
import org.fortiss.af3.exploration.alg.dse.module.IEvaluatorModule;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.opt4j.core.Objectives;
import org.opt4j.core.Value;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.ClassToInstanceMap;
import com.google.common.collect.MutableClassToInstanceMap;
import com.google.inject.Injector;

/**
 * Base class for Evaluators of {@link Phenotype}s.
 * 
 * @author diewald
 */
public class GuiceEvaluator extends CompositeEvaluator2<Double, ProvidingPhenotype> implements
		Evaluator<ProvidingPhenotype> {
	/** Flag to mark that marks the top-most evaluator in the composite structure. */
	protected boolean isTopLevelEvaluator = false;

	/** References the models provided by the {@link IExplorationTransformationService}. */
	protected ClassToInstanceMap<EObject> transformedModels = MutableClassToInstanceMap.create();

	/** Collection of required models demanded by the evaluators. */
	protected Collection<Class<? extends EObject>> requiredTransformedModelTypes;

	/** Top-most DSE {@link Injector}. */
	protected Injector dseInjector;

	/** Constructor. */
	public GuiceEvaluator(Injector dseInjector, Set<IEvaluatorModule<?, ?, ?>> subEvaluators) {
		isTopLevelEvaluator = true;
		this.dseInjector = dseInjector;
		this.subEvaluators = subEvaluators;
	}

	// /**
	// * Performs the initialization of {@code this} {@link CompositeEvaluator} that cannot be done
	// in
	// * the constructor, e.g., due to the missing registration of sub evaluators.
	// * It is executed when {@link #evaluateInternal(Phenotype)} is executed for the first
	// * time of this instance.
	// */
	// protected void init() {
	// if(isInitialized) {
	// return;
	// }
	// isInitialized = true;
	//
	// if(isTopLevelEvaluator) {
	// Set<IExplorationTargetEvaluator<?, ?, ?>> allSubEvaluators = new HashSet<>();
	// getAllSubEvaluators().forEach(
	// e -> allSubEvaluators.add(((EvaluatorWrapper<?, ?, ?>)e).getEvaluator()));
	//
	// @SuppressWarnings("unchecked") Collection<Class<? extends EObject>> tempReqModels =
	// getService(IExplorationEvaluatationService.class).getRequiredTransformedModels(
	// allSubEvaluators);
	// requiredTransformedModelTypes = tempReqModels;
	// }
	// }

	/**
	 * Internal method that calls the evaluators for {@link ComposableGenotype}s.
	 * <b>NOTE:</b> Sub-implementations must call {@code super.decode(P)}!!!
	 * 
	 * @param phenotype
	 *            {@link Phenotype} that shall be evaluated w.r.t. to the
	 *            {@link ExplorationSpecification}.
	 * @return Evaluation result as {@link Objectives}.
	 */
	@Override
	public Objectives evaluate(ProvidingPhenotype phenotype) {
		// ClassToInstanceMap<EObject> transformedModels = MutableClassToInstanceMap.create();
		// if(isTopLevelEvaluator && !requiredTransformedModelTypes.isEmpty()) {
		// try {
		// // TODO: Readd the transformation service to the evaluators.
		// // transformedModels =
		// // getService(IExplorationTransformationService.class).getTransformedModels(
		// // DSE_INTERNAL, null, phenotypeTypeMap.asExplorationEncoding(),
		// // systemModelAdapter.getInputModels(), requiredTransformedModelTypes,
		// // null);
		// } catch(Exception e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		// }
		this.evaluationInjector =
				dseInjector.createChildInjector(/* this, */phenotype.getGenotypeProviderModule(),
						phenotype.getPhenotypeProviderModule());

		// TODO: We have a 1:1 relation between ExplorationTargets and Objective(s). Returning the
		// Objectives map is a bad/confusing design and should be changed in Opt4J.
		Objectives objectives = new Objectives();

		for(IEvaluatorModule<Double, ?, ?> subEvaluator : getSubEvaluators()) {
			subEvaluator.setEvaluationInjector(evaluationInjector);
			Value<Double> resValue = subEvaluator.evaluate();
			objectives.add(subEvaluator.getObjective(), resValue);
		}

		return objectives;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	// TODO: avoid the need for suppressing the unchecked warning.
			public
			Set<IEvaluatorModule<Double, ?, ?>> getSubEvaluators() {
		return (Set<IEvaluatorModule<Double, ?, ?>>)(Set<?>)subEvaluators;
	}
}
