/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Decoder;

/**
 * Container for exceptions thrown in the decoding steps.
 * 
 * @author diewald
 */
public class DecodingException extends ExplorationException {
	/** Constructor specialized for decoding errors. */
	public DecodingException(Genotype genotype, Decoder<?, ?> decoder) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				".");
	}

	/** Constructor specialized for decoding errors. Takes an additional message. */
	public DecodingException(Genotype genotype, Decoder<?, ?> decoder, String msg) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				". " + msg);
	}

	/** Constructor specialized for decoding errors. Takes an additional message. */
	public DecodingException(Genotype genotype, Decoder<?, ?> decoder, String msg, Exception e) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				". " + msg, e);
	}

	/** Constructor specialized for decoding errors. */
	public DecodingException(Genotype genotype, ComposableDecoder<?, ?> decoder) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				".");
	}

	/** Constructor specialized for decoding errors. Takes an additional message. */
	public DecodingException(Genotype genotype, ComposableDecoder<?, ?> decoder, String msg) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				". " + msg);
	}

	/** Constructor specialized for decoding errors. Takes an additional message. */
	public DecodingException(Genotype genotype, ComposableDecoder<?, ?> decoder, String msg,
			Exception e) {
		super("Decoding Error: Could not decode " + genotype + " using the decoder " + decoder +
				". " + msg, e);
	}
}
