/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.opt4j.operators.crossover.Pair;

/**
 * Applies the crossover operation on the allocations of {@link ITaskAdapter}s to
 * {@link IExecutionUnitAdapter}s. It requires two parent {@link TaskMappingEncoding}s for which a
 * single allocation is randomly swapped to generate two new children {@link TaskMappingEncoding}s.
 * 
 * @author diewald
 */
public class CrossoverSingleAllocation<S extends TaskMappingEntry<ITaskAdapter<?>, Partition>, T extends TaskMappingEncoding<ITaskAdapter<?>, S>>
		extends CrossoverAllocationBase<S, T> {
	/** Constructor that allows to set the cross-over rate */
	public CrossoverSingleAllocation(double rate) {
		super(rate);
	}

	/** {@inheritDoc} */
	@Override
	public Pair<T> crossover(T encoding1, T encoding2) {
		@SuppressWarnings("unchecked") T newEncoding1 = (T)encoding1.clone();
		@SuppressWarnings("unchecked") T newEncoding2 = (T)encoding2.clone();

		int deplCompSize = encoding1.getRequesters().toArray().length;
		int randomIdx = rand.nextInt(deplCompSize - 1);
		ITaskAdapter<?> randomlySelectedComponent =
				(ITaskAdapter<?>)encoding1.getRequesters().toArray()[randomIdx];

		// TODO: Improve Crossover operator to respect allocation constraints.
		// boolean didCrossover = false;
		// int randomIdx = initRandomIdx;
		// for(int cnt = 0; !didCrossover && cnt <
		// systemModelAdapter.getDeployableComponents().size(); cnt++) {
		//
		// if(newEncoding2.getAllowedAllocations(deployableComponent));
		// (IDeployableComponentAdapter)encoding1.getDeployedComponents().toArray()[randomIdx++];
		//
		// randomIdx++;
		// if(randomIdx == deplCompSize) {
		// randomIdx = 0;
		// }
		// }
		crossoverAllocation(randomlySelectedComponent, newEncoding1, newEncoding2);

		return new Pair<T>(newEncoding1, newEncoding2);
	}
}
