/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;
import org.fortiss.af3.partition.model.Partition;
import org.fortiss.af3.partition.model.allocation.TaskToPartitionAllocationEntry;
import org.fortiss.af3.task.model.Task;

/**
 * Transforms a {@link TaskMappingEncoding} to a {@link SuperSet} of AF3
 * {@link TaskToPartitionAllocationEntry}s.
 * 
 * @author diewald
 */
public class TaskToPartitionTransformation extends
		SuperSetTransformationBase<TaskToPartitionAllocationEntry> {

	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> SuperSet<TaskToPartitionAllocationEntry>
			transform(TransformationState state, TransformationContext context,
					ExplorationTransformationInputs inputs,
					SubClassToInstanceMap<N> transformedModels, EObject outputModel) {
		FailSilentTaskMappingEncoding taskMapping =
				inputs.getEncoding(FailSilentTaskMappingEncoding.class);
		SuperSet<TaskToPartitionAllocationEntry> allocations =
				createSuperSet("TaskToPart", TaskToPartitionAllocationEntry.class);
		for(InstantiatedTaskMappingEntry entry : taskMapping.getMappingEntries()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					(Partition)encEntryModelElemAssoc.get(entry.getTarget(),
							org.fortiss.af3.partition.model.Partition.class);
			assert (af3Partition != null) : "Error in the mapping of internal Partitions --> AF3 Partitions.";
			TaskToPartitionAllocationEntry af3AllocEntry =
					createAllocationEntry(TaskToPartitionAllocationEntry.class);
			af3AllocEntry.getSourceElements().add((Task)entry.getSource().getObject());
			af3AllocEntry.setTargetElement(af3Partition);
			allocations.getEntries().add(af3AllocEntry);
		}
		return allocations;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(FailSilentTaskMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<TaskToPartitionAllocationEntry> getOutputType() {
		return TaskToPartitionAllocationEntry.class;
	}
}
