/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.partition;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import org.fortiss.af3.allocation.model.AllocationTableCollection;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.base.TransformationStrategyBase;

/**
 * Responsible for generating a {@link Deployment} that is compatible with the Deployment-plugin.
 * Thus, a Component -> Platform mapping resulting form the DSE can viewed and edited within the
 * {@link Deployment} editor.
 * 
 * @author diewald
 */
public class PartitionTransformationStrategy
		extends TransformationStrategyBase<AllocationTableCollection> {
	/** {@inheritDoc} */
	@Override
	public Class<AllocationTableCollection> getOutputType() {
		return AllocationTableCollection.class;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		// inputTypes.add(ComponentArchitecture.class);
		// inputTypes.add(PlatformArchitecture.class);
		// inputTypes.add(InstantiatedTaskMappingEncoding.class);
		// inputTypes.add(PartitionMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<? extends ITransformationModule<?>>> getParentModules() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isOptional() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public TransformationState getActivationStage() {
		return TransformationState.INITIALIZING;
	}
}
