/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.module;

import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Creator;

import com.google.inject.AbstractModule;
import com.google.inject.TypeLiteral;

/**
 * {@link Creator} module whose dependencies are defined using guice. Used to define sub-problems
 * of a DSE process.
 * 
 * @author diewald
 */
public abstract class CreatorModule<G extends Genotype> extends AbstractModule implements
		Creator<G> {

	/** Holds the type of the {@link Genotype}s generated by {@code this} CreatorModule. */
	TypeLiteral<G> genotypeType = new TypeLiteral<G>() {/**/
	};

	/**
	 * Returns the {@link Genotype} class whose instances are created by {@code this}
	 * {@link CreatorModule}.
	 */
	@SuppressWarnings("unchecked")
	public Class<G> getCreatedGenotypeType() {
		return (Class<G>)genotypeType.getRawType();
	}
}
