/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.AbstractTaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.FailSilentTaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.comm.MessageEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskinstantiation.MutateTaskInstanceModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateTaskMappingModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.comm.MessageDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.AbstractTaskMappingIdentityDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate.FailSilentAbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate.TaskInstanceResourceAlignmentDecoder;

/**
 * 
 * @author diewald
 */
public class TaskMappingExploration extends ExplorationFeature {

	/** Constructor. */
	// FIXME: Split up in std task mapping and the instantiated one (Fail-Silent).
	public TaskMappingExploration() {
		super("Task Mapping Exploration");

		// Variables
		addVariable(AbstractTaskMappingEncoding.class, new AbstractTaskMappingCreator(),
				new MutateTaskMappingModule());
		addVariable(FailSilentTaskMappingEncoding.class, new FailSilentTaskMappingCreator(),
				new MutateTaskInstanceModule());

		// Decoding
		addProblem(AbstractTaskMappingEncoding.class, new AbstractTaskMappingIdentityDecoder());
		addProblem(FailSilentTaskMappingEncoding.class, new FailSilentAbstractTaskMappingDecoder());
		// getService(IExplorationModuleService.class).registerProblem(
		// FaultDetectionVotingTaskMappingEncoding.class,
		// new FaultDetectionVotingAbstractTaskMappingDecoder());
		addProblem(InstantiatedTaskMappingEncoding.class,
				new TaskInstanceResourceAlignmentDecoder());
		addProblem(MessageEncoding.class, new MessageDecoder());
	}
}
