/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.moea.model.predefined.impl;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.dsl.model.expressions.ArithmeticExpression;
import org.fortiss.af3.exploration.dsl.model.expressions.ObjectiveExpression;
import org.fortiss.af3.exploration.moea.model.predefined.FailureMinObjective;
import org.fortiss.af3.exploration.moea.model.predefined.ReliabilityAnalysis;

/**
 * Static method implementations for {@link FailureMinObjective}s.
 * 
 * @author diewald
 */
public class FailureMinObjectiveStaticImpl {

	/**
	 * Static implementation of the {@link FailureMinObjective#setTargetComponent(Component)}
	 * method.
	 */
	public static void setFailureMinTargetComponent(FailureMinObjective objective,
			Component targetComponent) {
		if(objective == null || targetComponent == null) {
			return;
		}
		// The targetComponent is set in the ReliabilityAnalysis expression.
		ReliabilityAnalysis relAnalysis = getReliabilityAnalysis(objective);
		if(relAnalysis != null) {
			relAnalysis.setModelElement(targetComponent);
		}
	}

	/** Static implementation of the {@link FailureMinObjective#getTargetComponent()} method. */
	public static Component getFailureMinTargetComponent(FailureMinObjective objective) {
		if(objective == null) {
			return null;
		}
		// The targetComponent is set in the ReliabilityAnalysis expression.
		ReliabilityAnalysis relAnalysis = getReliabilityAnalysis(objective);
		if(relAnalysis != null) {
			return relAnalysis.getModelElement();
		}
		return null;
	}

	/**
	 * Returns the {@link ReliabilityAnalysis} expression contained in a {@link FailureMinObjective}
	 * .
	 */
	public static ReliabilityAnalysis getReliabilityAnalysis(FailureMinObjective objective) {
		if(objective == null) {
			return null;
		}
		// Walk the expressions until the ReliabilityAnalysis is found (hardcoded).
		if(objective.getMoeaExpression() instanceof ObjectiveExpression) {
			ObjectiveExpression expr = (ObjectiveExpression)objective.getMoeaExpression();
			ArithmeticExpression<?> aExpr = expr.getRight();
			if(aExpr instanceof ReliabilityAnalysis) {
				return (ReliabilityAnalysis)aExpr;
			}
		}
		return null;
	}
}
